import { Socket, io } from "socket.io-client";
import { Result } from "../helper/result";
import { TypedEvent } from "../helper/typed_event";
import { RuntimeModel } from "../../features/behavior_tree_builder/presentation/ui/actions/runtime_actions";

export class SocketRepository extends TypedEvent<any> {
  serverURL = "ws://localhost:4001";
  socket: Socket | undefined;

  async connect(): Promise<Result<boolean, boolean>> {
    const socket = io(this.serverURL);

    this.socket = socket;
    socket.connect();
    socketCoreInstances.forEach((el) => {
      socket.on(el.event, (event) => el.emit(event))
    })
    socket.on('realtime', (d) => {
      this.emit({
        event: "realtime",
        payload: d
      })
    })
    if (socket.connected) {
      return Result.ok(true)
    }
    return Result.error(false)
  }

}

export const socketRepository = new SocketRepository()


export abstract class SocketCore<T> extends TypedEvent<T> {
  abstract event: string

}
export class RunTimeSocketRepository extends SocketCore<RuntimeModel> {
  event = 'realtimeV2';
}
export const runTimeSocketRepository = new RunTimeSocketRepository();
const socketCoreInstances: SocketCore<any>[] = [runTimeSocketRepository]