import React, { useState } from "react";
import { observer } from "mobx-react-lite";
import { IFormBuilderComponentsProps } from "../../form_builder_components";
import { useStore } from "../../../../../helper/use_store";
import { SelectProcessStore } from "./select_process_store";
import { useEffect } from "react";
import { SelectProcessModel } from "../model/select_process_model";
import { Loader } from "../../../../loader/loader";
import { CoreSelect } from "../../../../select/select";
import { message } from "antd";
import { CalculationModel } from "../../../../../../features/calculation_instance/model/calculation_model";

export const SelectProcess = observer((props: IFormBuilderComponentsProps<SelectProcessModel>) => {
  const [store] = useState(new SelectProcessStore());
  useEffect(() => {
    if (typeof props.dependency === "string") {
      store.loadClassInstance(SelectProcessModel, JSON.parse(props.dependency));
    } else {
      store.loadClassInstance(SelectProcessModel, props.dependency);
    }

    store.init();
  }, []);
  return (
    <div>
      {store.isLoading ? (
        <Loader />
      ) : (
        <div>
          <CoreSelect
            items={store.calculationInstances.map((el) => el.instanceName)}
            value={store.viewModel?.selectProcess?.value.instanceName ?? ""}
            label={`Процесс тип ${store?.viewModel?.type ?? ""}`}
            onChange={async (value: string, index: number) => {
              store.updateForm({
                selectProcess: { value: store.calculationInstances.at(index) ?? CalculationModel.empty() },
              });
              (await store.viewModel.valid<SelectProcessModel>()).fold(
                (model) => props.onChange(model),
                (error) => message.error(error)
              );
            }}
          />
        </div>
      )}
    </div>
  );
});
