import * as React from "react";
import { useRete } from "rete-react-plugin";
import { createEditor } from "./ui/editor/editor";
import { SkillTree } from "./ui/skill_tree/skill_tree";
import { BehaviorTreeBuilderStore, DrawerState } from "./behavior_tree_builder_store";
import { observer } from "mobx-react-lite";
import { Icon } from "../../../core/ui/icons/icons";
import { CoreText, CoreTextType } from "../../../core/ui/text/text";
import { useNavigate, useParams } from "react-router-dom";
import { IForms, forms } from "./ui/forms/forms";
import { themeStore } from "../../..";
import { MainPageV2 } from "../../../core/ui/pages/main_page_v2";
import { DrawerV2 } from "../../../core/ui/drawer/drawer";
import { Panel, PanelGroup, PanelResizeHandle } from "react-resizable-panels";
import PopoverV2 from "../../../core/ui/popover/popover";

export const behaviorTreeBuilderScreenPath = "behavior/tree/screen/path";

export interface DOMReact {
  x: number;
  y: number;
  width: number;
  height: number;
  top: number;
  right: number;
  bottom: number;
  left: number;
}

export const behaviorTreeBuilderStore = new BehaviorTreeBuilderStore();

export const BehaviorTreeBuilderPath = (id?: string) => {
  if (id) {
    return `/behavior/tree/${id}`;
  }
  return `/behavior/tree/`;
};

export const isBtScreen = () =>
  Boolean(window.location.href.match(new RegExp(BehaviorTreeBuilderPath()))?.at(0)) ?? false;

export const BehaviorTreeBuilderScreen = observer(() => {
  const navigate = useNavigate();
  const { id } = useParams();
  // @ts-expect-error
  const [ref] = useRete<HTMLDivElement>(createEditor);
  const store = behaviorTreeBuilderStore;
  React.useEffect(() => {
    if (ref.current) {
      // @ts-expect-error
      const domReact: DOMReact = ref.current.getBoundingClientRect();

      // УБЕРИ + 300
      store.dragZoneSetOffset(0, domReact.y, domReact.width + 300, domReact.height);
    }
  }, [ref.current]);

  React.useEffect(() => {
    store.init(navigate).then(() => store.initParam(id));
    return () => {
      store.dispose();
    };
  }, []);
  return (
    <MainPageV2
      style={{ position: "absolute", height: "100%", overflow: "hidden" }}
      bgColor={themeStore.theme.black}
      children={
        <>
          <>
            <div style={{ position: "absolute", zIndex: 100 }}>
              <div
                style={{
                  width: "100%",
                  display: "flex",
                  flexDirection: "row",
                  justifyContent: "end",
                  paddingRight: 30,
                  alignItems: "center",
                }}
              >
                <div
                  style={{
                    backgroundColor: themeStore.theme.greenWhite,
                    height: 40,
                    textAlign: "center",
                    alignContent: "center",
                    width: 40,
                    borderRadius: 100,
                  }}
                >
                  <Icon style={{ height: 21 }} onClick={() => store.onClickSaveBehaviorTree()} type="Floppy" />
                </div>
              </div>
            </div>
            <div
              style={{
                height: "100%",
                width: 300,
                zIndex: 10,
                position: "absolute",
                display: "flex",
                flexDirection: "column",
                justifyContent: "space-between",
                backgroundColor: themeStore.theme.surfaceContainerLow,
              }}
            >
              <div style={{ margin: 20 }}>
                <div style={{ height: 58, alignContent: "center" }}>
                  <CoreText
                    text="Поведение"
                    type={CoreTextType.mediumV2}
                    color={themeStore.theme.onSurfaceVariant}
                    style={{ marginLeft: 20 }}
                  />
                  <div style={{ width: "100%", height: 1, backgroundColor: themeStore.theme.outlineVariant }} />
                </div>
                {store.skillTemplates ? <SkillTree skills={store.skillTree} dragEnd={store.dragEnd} /> : null}
              </div>
            </div>

            <div
              style={{
                position: "absolute",
                left: 300,
                height: "100%",
                width: "calc(100% - 300px)",
              }}
            >
              <PanelGroup direction="horizontal" style={{ width: "100%" }}>
                <Panel>
                  <div
                    ref={ref}
                    className="dotted"
                    style={{
                      width: "100%",
                      height: "100%",
                      backgroundSize: "20px 20px",
                    }}
                  />
                </Panel>
                {store.panels.map((el, index) => (
                  <>
                    <PanelResizeHandle
                      children={
                        <>
                          <div style={{ width: 10, height: "100%", backgroundColor: "beige" }}></div>
                        </>
                      }
                    ></PanelResizeHandle>

                    <Panel
                      style={{ backgroundColor: "rgb(29, 27, 32)" }}
                      defaultSize={el.size}
                      onResize={(size) => store.panelResize(size, index)}
                    >
                      <div
                        style={{
                          display: "flex",
                          width: "100%",
                          justifyContent: "space-between",
                          backgroundColor: "beige",
                        }}
                      >
                        <PopoverV2
                          children={<Icon type={"3points"} height={26} />}
                          content={
                            <div style={{ width: "max-content", padding: 10 }}>
                              {store.panelActions.map((el, i) => (
                                <CoreText type={CoreTextType.medium} text={el.name} onClick={() => el.action(index)} />
                              ))}
                            </div>
                          }
                        />
                        <div style={{ color: "black", alignContent: "center", paddingRight: 10 }}>{el.name}</div>
                      </div>
                      {el.body}
                    </Panel>
                  </>
                ))}
              </PanelGroup>
            </div>
          </>

          <DrawerV2
            title={store.titleDrawer}
            onClose={() => store.editDrawer(DrawerState.editThreadBehaviorTree, false)}
            isOpen={store.drawers.find((el) => el.name === DrawerState.editThreadBehaviorTree)?.status}
            width={window.innerWidth / 2}
          >
            <div
              style={{
                display: "flex",
                flexDirection: "column",
                justifyContent: "space-between",
                height: "100%",
                overflow: "auto",
              }}
            >
              <div style={{ height: "100%" }}>
                {store.skillTemplates?.getForms(store.selected ?? "").map((formType, index) =>
                  forms(
                    store.filledOutTemplates?.getDependencyBySkillLabelAndType(
                      forms(null, () => {}, store).find((form) => form.name.isEqual(formType))?.name ?? "",
                      store.selectedSid ?? ""
                    ),
                    (dependency) => store.formUpdateDependency(dependency, formType),
                    store
                  )
                    .rFind<IForms>((form) => form.name.isEqual(formType))
                    .fold(
                      (s) => {
                        return (
                          <div key={index} style={{ flex: 1 }}>
                            {s.component}
                          </div>
                        );
                      },
                      () => (
                        <div key={index + "error"} style={{ height: "100%" }}>
                          Error: Unknown form type {formType}
                        </div>
                      )
                    )
                )}
              </div>
            </div>
          </DrawerV2>
        </>
      }
    />
  );
});
