import { NavigateFunction, useParams } from "react-router-dom";
import { CoreHttpRepository, HttpMethod } from "../../../../../core/repository/core_http_repository";
import { CoreError, UiErrorState } from "../../../../../core/store/base_store";
import { ProcessStatus } from "../../../../dataset/dataset_model";
import makeAutoObservable from "mobx-store-inheritance";
import { useStore } from "../../../../../core/helper/use_store";
import { Loader } from "../../../../../core/ui/loader/loader";
import { useEffect } from "react";
import {
  RunTimeSocketRepository,
  runTimeSocketRepository,
} from "../../../../../core/repository/core_socket_repository";
import { observer } from "mobx-react-lite";
interface IPid {
  pid: string;
}
export interface RuntimeModel {
  [name: string]: { pid: number; status: String };
}

export class RunTimeHttpRepository extends CoreHttpRepository {
  feature = "/run_time";
  stop = (model: IPid) => this._jsonRequest(HttpMethod.POST, this.feature + "/kill", model);
  getStatuses = () => this._jsonRequest<RuntimeModel>(HttpMethod.GET, this.feature + "/status");
  execBt = (id: string) => this._jsonRequest(HttpMethod.POST, this.feature + `/exec/bt?id=${id}`);
}
// export class RunTimeSocketRepository extends SockeCore {}
export class RunTimeStore extends UiErrorState<CoreError> {
  runTimeHttpRepository: RunTimeHttpRepository = new RunTimeHttpRepository();
  runTimeSocketRepository: RunTimeSocketRepository = runTimeSocketRepository;
  pageId: string;
  runTime?: RuntimeModel = {};
  constructor() {
    super();
    makeAutoObservable(this);
    runTimeSocketRepository.on((event) => (console.log(event), (this.runTime = event)));
  }
  async init(navigate?: NavigateFunction | undefined): Promise<any> {
    this.mapOk("runTime", this.runTimeHttpRepository.getStatuses());
    this.navigate = navigate;
  }

  executeBt = () => this.runTimeHttpRepository.execBt(this.pageId);
  initParam = (id: string) => (this.pageId = id);
  isExecuteBt = (): boolean => {
    if (this.runTime === undefined) {
      return false;
    }
    if (
      this.runTime["EXEC_BT"] !== undefined &&
      this.runTime["EXEC_BT"]["status"] !== undefined &&
      this.runTime["EXEC_BT"]["status"] === "endOk"
    ) {
      return false;
    }
    return true;
  };
}
export const RunTimeActions = observer(() => {
  const store = useStore(RunTimeStore);
  const params = useParams();
  useEffect(() => {
    store.initParam(params.id as string);
  }, []);
  return (
    <div style={{ width: "100%" }}>
      {store.isLoading ? (
        <Loader />
      ) : (
        <>
          {store.isExecuteBt() ? (
            <>
              <div
                style={{ justifyContent: "center", color: "white", justifySelf: "center" }}
                onClick={() => store.executeBt()}
              >
                Дерево запущено
              </div>
            </>
          ) : (
            <>
              <div
                style={{ justifyContent: "center", color: "white", justifySelf: "center" }}
                onClick={() => store.executeBt()}
              >
                Запустить дерево
              </div>
            </>
          )}
        </>
      )}
    </div>
  );
});
