import React from "react";
import { observer } from "mobx-react-lite";
import { TopicsStore } from "./topics_store";
import { useStore } from "../../core/helper/use_store";
import { CoreButton } from "../../core/ui/button/button";
import { CoreModal } from "../../core/ui/modal/modal";
import { InputV2 } from "../../core/ui/input/input_v2";
import { CoreText, CoreTextType } from "../../core/ui/text/text";
import { CoreInput } from "../../core/ui/input/input";

export const TopicsScreenPath = "/topics";

export const TopicsScreen = observer(() => {
  const store = useStore(TopicsStore);

  return (
    <>
      <div style={{ display: "flex" }}>
        <CoreButton text="добавить новый топик" style={{ width: 200 }} onClick={() => store.modalShow()} />
        <CoreButton text="импорт топика" style={{ width: 200 }} onClick={() => store.openTopicModal()} />
      </div>
      {store.topics?.map((el) => (
        <div style={{ margin: 10, border: "1px solid red" }}>
          <CoreText text="Delete" type={CoreTextType.header} onClick={() => store.deleteTopic(el._id ?? "")} />
          <CoreText
            text="Copy"
            type={CoreTextType.header}
            onClick={() => window.prompt("Copy to clipboard: Ctrl+C, Enter", JSON.stringify(el))}
          />

          <div>{el.name}</div>
          <div>{el.type}</div>
        </div>
      ))}
      <CoreModal
        isOpen={store.isModalOpen}
        onClose={() => store.modalCancel()}
        children={
          <div>
            <InputV2 label={"type"} onChange={(text) => store.updateForm({ type: text })} />
            <InputV2 label={"name"} onChange={(text) => store.updateForm({ name: text })} />
            <CoreButton text="new" onClick={() => store.createTopic()} />
          </div>
        }
      />
      <CoreModal
        isOpen={store.importTopicModal}
        onClose={() => store.cancelImportTopicModal()}
        children={
          <div>
            <InputV2 onChange={(text) => store.importTopic(text)} label="import" />
          </div>
        }
      />
    </>
  );
});
