import makeAutoObservable from "mobx-store-inheritance";
import { FormState } from "../../core/store/base_store";
import { HttpError } from "../../core/repository/core_http_repository";
import { ITopicModel, TopicViewModel as TopicModel } from "./topic_view_model";
import { NavigateFunction } from "react-router-dom";
import { TopicsHttpRepository } from "./topics_repository";

export class TopicsStore extends FormState<TopicModel, HttpError> {
  importTopic(text: string): void {
    this.loadClassInstance(TopicModel, JSON.parse(text))

    this.createTopic()
  }


  viewModel: TopicModel = TopicModel.empty();
  topics?: ITopicModel[];
  topicsHttpRepository: TopicsHttpRepository = new TopicsHttpRepository();
  importTopicModal = false;
  constructor() {
    super();
    makeAutoObservable(this);
  }
  init = async (navigate?: NavigateFunction | undefined) => {
    this.modalCancel();
    this.importTopicModal = false;
    await this.mapOk("topics", this.topicsHttpRepository.getAllTopics());
  };
  cancelImportTopicModal = () => {
    this.importTopicModal = false;
  }
  openTopicModal = () => {
    this.importTopicModal = true;
  }
  deleteTopic = async (id: string) => (await (this.topicsHttpRepository.deleteTopic(id))).map(() => this.init());
  createTopic = async () =>
    (await this.viewModel.validMessage<TopicModel>()).map(async (model) =>
      (await this.topicsHttpRepository.createTopics(model)).map(() => this.init())
    );
}
